@@ -3,10 +3,15 @@
 
    Gleb Beliakov, 2020
 */
-#include "generaldefs.h"
-#ifndef NO_R
+#include "generaldefs.h" //NH
 #define R_NO_REMAP
+
+// #define __R //NH
+#ifdef __R
 //#include "cpp11.hpp"
+
+
+
 #include <R.h>
 #include <Rdefines.h>
 #include <Rinternals.h>
@@ -14,11 +19,13 @@
 //#include <Rcpp.h>
 
 
-
 #include <stdlib.h>
-
 #include "wrapperpy.h"
 
+#ifdef PYTHON_
+#define LIBEXP
+#endif
+
 //#if (defined(__clang__) || defined(__APPLE__) || defined(__arm64__)) && defined(_LIBCPP_VERSION)
 //#if (defined(__clang__) ||  defined(__arm64__)) && defined(_LIBCPP_VERSION)	
 #if  defined(_LIBCPP_VERSION)	// may be just detect libc++
@@ -143,8 +150,8 @@ void releaseVectorWrapper( std::vector<T,  std::allocator<T> > volatile &targetV
 #endif
 
 
-// NH undef -> define 
-// #undef PYTHON_
+
+// #undef PYTHON_ commented out by NH
 
 extern "C" {
 
@@ -1961,6 +1968,63 @@ void fm_arraysizeCallR(int* n, int* m, int* kint, int* out, double* Rfactorials)
 	*out= fm_arraysize(*n, m1, *kint);
 }
 
+// generate fuzzy measures randomly using topological sort
+int generate_fm_tsortCall(int_64* num, int* n, int* kint, int* markov, int* option, double* K, double * vv, int* Rcard, int*  Rcardpos, double*  Rbit2card, double*  Rcard2bit, double* Rfactorials)
+{
+    card = Rcard;
+    cardpos = Rcardpos;
+    bit2card = (int_64*)Rbit2card;
+    card2bit = (int_64*)Rcard2bit;
+    m_factorials = Rfactorials;
+    return generate_fm_tsort(*num, *n, *kint, *markov, *option, *K, vv);
+}
+
+
+
+// generate convex (supermodular)  fuzzy measures randomly using topological sort
+int generate_fmconvex_tsortCall(int_64* num, int* n, int* kint, int* markov, int* option, double* K, double * vv, int* Rcard, int*  Rcardpos, double*  Rbit2card, double*  Rcard2bit, double* Rfactorials)
+{
+    card = Rcard;
+    cardpos = Rcardpos;
+    bit2card = (int_64*)Rbit2card;
+    card2bit = (int_64*)Rcard2bit;
+    m_factorials = Rfactorials;
+    int opt = (*option) | 8; // means we do rejections inside for 1000 steps
+    return generate_fmconvex_tsort(*num, *n, *kint, *markov, opt, *K,  vv);
+}
+
+// generate fuzzy measures randomly using MinimalsPlus method
+int generate_fm_minplusCall(int_64* num, int* n, int* kint, int* markov, int* option, double* K, double * vv, int* Rcard, int*  Rcardpos, double*  Rbit2card, double*  Rcard2bit, double* Rfactorials)
+{
+    card = Rcard;
+    cardpos = Rcardpos;
+    bit2card = (int_64*)Rbit2card;
+    card2bit = (int_64*)Rcard2bit;
+    m_factorials = Rfactorials;
+    return generate_fm_minplus(*num, *n, *kint, *markov, *option, *K, vv);
+}
+
+// generate simple 2 additive supermodular (convex) and submodular capacities
+// size is the returned size of each vector in compact representation (singletons and pairs only
+int generate_fm_2additive_convexCall(int_64* num, int* n, int * size, double * vv)
+{
+    return  generate_fm_2additive_convex(*num, *n,  size,  vv);
+}
+
+int generate_fm_2additive_concaveCall(int_64* num, int* n, int * size, double * vv)
+{
+    return  generate_fm_2additive_concave(*num, *n, size, vv);
+}
+
+int generate_fm_2additive_convex_withsomeindependentCall(int_64* num, int* n, int * size, double * vv)
+// as above, but resets randomly some interactions to 0
+{
+    return generate_fm_2additive_convex_withsomeindependent(*num, *n, size, vv);
+}
+
+
+
+#ifdef NH_USED_IN_R_INTERFACE 
 
 // generate fuzzy measures randomly using topological sort
 void generate_fm_tsortCall(int* num1, int* n, int* kint, int* markov, int* option, double* K, double * vv, int* Rcard, int*  Rcardpos, double*  Rbit2card, double*  Rcard2bit, double* Rfactorials)
@@ -2022,6 +2086,7 @@ void generate_fm_2additive_convex_withsomeindependentCall(int* num1, int* n, int
 	 generate_fm_2additive_convex_withsomeindependent(num, *n, size, vv);
 }
 
+#endif // NH_USED_IN_R_INTERFACE
 
 void export_maximal_chainsCall(int* n,  double * v, double * mc, double* Rfactorials)
 {
@@ -2072,25 +2137,30 @@ LIBEXP int py_fm_arraysize(int n, int kint, struct fm_env* env)
 LIBEXP int py_generate_fm_tsort(int num, int n, int kint, int markov, int option, double K, double * vv, struct fm_env* env)
 {
 	int_64 num1 = num;
-	generate_fm_tsortCall(&num, &n, &kint, &markov, &option, &K, vv, env->card, env->cardpos, env->bit2card, env->card2bit, env->factorials);
-	return num; //NH
+	// NH: change num to num1 in parameter list
+	generate_fm_tsortCall(&num1, &n, &kint, &markov, &option, &K, vv, env->card, env->cardpos, env->bit2card, env->card2bit, env->factorials);
+	return num;
 }
+
 LIBEXP int py_generate_fmconvex_tsort(int num, int n, int kint, int markov, int option, double K, double * vv, struct fm_env* env)
 {
-	generate_fmconvex_tsortCall(&num, &n, &kint, &markov, &option, &K, vv, env->card, env->cardpos, env->bit2card, env->card2bit, env->factorials);
-	return num; //NH
+	int_64 num1 = num;
+	// NH: change num to num1 in parameter list
+	return generate_fmconvex_tsortCall(&num1, &n, &kint, &markov, &option, &K, vv, env->card, env->cardpos, env->bit2card, env->card2bit, env->factorials);
 }
 LIBEXP int py_generate_fm_minplus(int num, int n, int kint, int markov, int option, double K, double * vv, struct fm_env* env)
 {
-	generate_fm_minplusCall(&num, &n, &kint, &markov, &option, &K, vv, env->card, env->cardpos, env->bit2card, env->card2bit, env->factorials);
-	return num; //NH
+	int_64 num1 = num;
+	// NH: change num to num1 in parameter list
+	return generate_fm_minplusCall(&num1, &n, &kint, &markov, &option, &K, vv, env->card, env->cardpos, env->bit2card, env->card2bit, env->factorials);
 }
 
 LIBEXP int py_generate_fm_2additive_convex(int num, int n,  double * vv)
 {
 	int_64 num1 = num;
 	int size;
-	generate_fm_2additive_convexCall(&num, &n, &size, vv);
+	// NH: change num to num1 in parameter list
+	generate_fm_2additive_convexCall(&num1, &n, &size, vv);
 	return size;
 }
 
@@ -2098,7 +2168,8 @@ LIBEXP int py_generate_fm_2additive_concave(int num, int n, double * vv)
 {
 	int_64 num1 = num;
 	int size;
-	generate_fm_2additive_concaveCall(&num, &n, &size, vv);
+	// NH: change num to num1 in parameter list
+	generate_fm_2additive_concaveCall(&num1, &n, &size, vv);
 	return size;
 }
 
@@ -2106,7 +2177,8 @@ LIBEXP int py_generate_fm_2additive_convex_withsomeindependent(int num, int n,
 {
 	int_64 num1 = num;
 	int size;
-	generate_fm_2additive_convex_withsomeindependentCall(&num, &n, &size, vv);
+	// NH: change num to num1 in parameter list
+	generate_fm_2additive_convex_withsomeindependentCall(&num1, &n, &size, vv);
 	return size;
 }
 
